@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is a kernel build script for Fedora 10's 2.6.27 kernel.
+# This is a kernel build script for Fedora 12's 2.6.31 kernel.
 #
 
 die () {
@@ -10,33 +10,33 @@ die () {
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.27.30-170.2.82.fc10.src.rpm ]
+if [ ! -r kernel-2.6.31.5-127.fc12.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/fedora/updates/10/SRPMS/kernel-2.6.27.30-170.2.82.fc10.src.rpm || die "Can't download source package."
+    wget http://ftp.riken.jp/Linux/fedora/development/source/SRPMS/kernel-2.6.31.5-127.fc12.src.rpm || die "Can't download source package."
 fi
-rpm -ivh kernel-2.6.27.30-170.2.82.fc10.src.rpm || die "Can't install source package."
+rpm -ivh kernel-2.6.31.5-127.fc12.src.rpm || die "Can't install source package."
 
 cd /root/rpmbuild/SOURCES/ || die "Can't chdir to /root/rpmbuild/SOURCES/ ."
-if [ ! -r ccs-patch-1.7.0-20090911.tar.gz ]
+if [ ! -r ccs-patch-1.7.1-20091111.tar.gz ]
 then
-    wget http://osdn.dl.sourceforge.jp/tomoyo/43375/ccs-patch-1.7.0-20090911.tar.gz || die "Can't download patch."
+    wget http://osdn.dl.sourceforge.jp/tomoyo/43375/ccs-patch-1.7.1-20091111.tar.gz || die "Can't download patch."
 fi
 
-cd /tmp/ || die "Can't chdir to /tmp/ ."
-cp -p /root/rpmbuild/SPECS/kernel.spec . || die "Can't copy spec file."
+cd /root/rpmbuild/SPECS/ || die "Can't chdir to /root/rpmbuild/SPECS/ ."
+cp -p kernel.spec ccs-kernel.spec || die "Can't copy spec file."
 patch << "EOF" || die "Can't patch spec file."
---- kernel.spec	2009-08-17 21:00:35.000000000 +0900
-+++ kernel.spec	2009-08-23 20:37:16.529989199 +0900
+--- ccs-kernel.spec	2009-11-08 10:46:36.000000000 +0900
++++ ccs-kernel.spec	2009-11-10 09:27:37.114308507 +0900
 @@ -15,7 +15,7 @@
  # that the kernel isn't the stock distribution kernel, for example,
  # by setting the define to ".local" or ".bz123456"
  #
--#% define buildid .local
-+%define buildid _tomoyo_1.7.0
+-# % define buildid .local
++%define buildid _tomoyo_1.7.1
  
  # fedora_build defines which build revision of this kernel version we're
  # building. Rather than incrementing forever, as with the prior versioning
-@@ -387,6 +387,11 @@
+@@ -409,6 +409,11 @@
  # to versions below the minimum
  #
  
@@ -48,16 +48,16 @@ patch << "EOF" || die "Can't patch spec file."
  #
  # First the general kernel 2.6 required versions as per
  # Documentation/Changes
-@@ -425,7 +430,7 @@
+@@ -444,7 +449,7 @@
  # Packages that need to be installed before the kernel is, because the %post
  # scripts use them.
  #
--%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, mkinitrd >= 6.0.61-1, kernel-firmware >= %{rpmversion}-%{pkg_release}
-+%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, mkinitrd >= 6.0.61-1
- 
- #
- # This macro does requires, provides, conflicts, obsoletes for a kernel package.
-@@ -453,7 +458,7 @@
+-%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, kernel-firmware >= %{rpmversion}-%{pkg_release}, grubby >= 7.0.4-1
++%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, grubby >= 7.0.4-1
+ %if %{with_dracut}
+ %define initrd_prereq  dracut >= 001-7
+ %else
+@@ -480,7 +485,7 @@
  AutoProv: yes\
  %{nil}
  
@@ -66,7 +66,7 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  URL: http://www.kernel.org/
-@@ -882,7 +887,7 @@
+@@ -914,7 +919,7 @@
  Provides: kernel-devel-uname-r = %{KVERREL}%{?1:.%{1}}\
  AutoReqProv: no\
  Requires(pre): /usr/bin/find\
@@ -75,18 +75,18 @@ patch << "EOF" || die "Can't patch spec file."
  This package provides kernel headers and makefiles sufficient to build modules\
  against the %{?2:%{2} }kernel package.\
  %{nil}
-@@ -1425,6 +1430,10 @@
+@@ -1516,6 +1521,10 @@
  
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.7.0-20090911.tar.gz
-+patch -sp1 < patches/ccs-patch-2.6.27-fedora-10.diff
++tar -zxf %_sourcedir/ccs-patch-1.7.1-20091111.tar.gz
++patch -sp1 < patches/ccs-patch-2.6.31-fedora-12.diff
 +
  %endif
  
  # Any further pre-build tree manipulations happen here.
-@@ -1451,6 +1460,9 @@
+@@ -1542,6 +1551,9 @@
  for i in *.config
  do
    mv $i .config
@@ -97,11 +97,10 @@ patch << "EOF" || die "Can't patch spec file."
    make ARCH=$Arch %{oldconfig_target} > /dev/null
    echo "# $Arch" > configs/$i
 EOF
-mv kernel.spec ccs-kernel.spec || die "Can't rename spec file."
 echo ""
 echo ""
 echo ""
-echo "Edit /tmp/ccs-kernel.spec if needed, and run"
-echo "rpmbuild -bb /tmp/ccs-kernel.spec"
+echo "Edit /root/rpmbuild/SPECS/ccs-kernel.spec if needed, and run"
+echo "rpmbuild -bb /root/rpmbuild/SPECS/ccs-kernel.spec"
 echo "to build kernel rpm packages."
 exit 0
