@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2009  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/06
+ * Version: 1.7.1   2009/11/11
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -22,11 +22,6 @@
 #endif
 #include "internal.h"
 
-/* For compatibility with older kernels. */
-#ifndef for_each_process
-#define for_each_process for_each_task
-#endif
-
 /* Variables definitions.*/
 
 /* The initial domain. */
@@ -696,7 +691,9 @@ static int ccs_environ(struct ccs_execve_entry *ee)
 {
 	struct ccs_request_info *r = &ee->r;
 	struct linux_binprm *bprm = ee->bprm;
-	char *arg_ptr = ee->tmp;
+	/* env_page->data is allocated by ccs_dump_page(). */
+	struct ccs_page_dump env_page = { };
+	char *arg_ptr; /* Size is CCS_EXEC_TMPSIZE bytes */
 	int arg_len = 0;
 	unsigned long pos = bprm->p;
 	int offset = pos % PAGE_SIZE;
@@ -706,14 +703,16 @@ static int ccs_environ(struct ccs_execve_entry *ee)
 	int error = -ENOMEM;
 	if (!r->mode || !envp_count)
 		return 0;
+	arg_ptr = kzalloc(CCS_EXEC_TMPSIZE, GFP_KERNEL);
+	if (!arg_ptr)
+		goto out;
 	while (error == -ENOMEM) {
-		if (!ccs_dump_page(bprm, pos, &ee->dump))
+		if (!ccs_dump_page(bprm, pos, &env_page))
 			goto out;
 		pos += PAGE_SIZE - offset;
 		/* Read. */
 		while (argv_count && offset < PAGE_SIZE) {
-			const char *kaddr = ee->dump.data;
-			if (!kaddr[offset++])
+			if (!env_page.data[offset++])
 				argv_count--;
 		}
 		if (argv_count) {
@@ -721,8 +720,7 @@ static int ccs_environ(struct ccs_execve_entry *ee)
 			continue;
 		}
 		while (offset < PAGE_SIZE) {
-			const char *kaddr = ee->dump.data;
-			const unsigned char c = kaddr[offset++];
+			const unsigned char c = env_page.data[offset++];
 			if (c && arg_len < CCS_EXEC_TMPSIZE - 10) {
 				if (c == '=') {
 					arg_ptr[arg_len++] = '\0';
@@ -758,6 +756,7 @@ static int ccs_environ(struct ccs_execve_entry *ee)
  out:
 	if (r->mode != 3)
 		error = 0;
+	kfree(env_page.data);
 	return error;
 }
 
@@ -862,56 +861,6 @@ static int ccs_get_root_depth(void)
 	return depth;
 }
 
-static LIST_HEAD(ccs_execve_list);
-DEFINE_SPINLOCK(ccs_execve_list_lock);
-unsigned int ccs_in_execve_counter;
-
-/**
- * ccs_allocate_execve_entry - Allocate memory for execve().
- *
- * Returns pointer to "struct ccs_execve_entry" on success, NULL otherwise.
- */
-static inline struct ccs_execve_entry *ccs_allocate_execve_entry(void)
-{
-	struct ccs_execve_entry *ee = kzalloc(sizeof(*ee), GFP_KERNEL);
-	if (!ee)
-		return NULL;
-	ee->tmp = kzalloc(CCS_EXEC_TMPSIZE, GFP_KERNEL);
-	if (!ee->tmp) {
-		kfree(ee);
-		return NULL;
-	}
-	ee->reader_idx = ccs_read_lock();
-	/* ee->dump->data is allocated by ccs_dump_page(). */
-	ee->task = current;
-	ee->previous_domain = ee->task->ccs_domain_info;
-	spin_lock(&ccs_execve_list_lock);
-	ccs_in_execve_counter++;
-	list_add(&ee->list, &ccs_execve_list);
-	spin_unlock(&ccs_execve_list_lock);
-	return ee;
-}
-
-/**
- * ccs_free_execve_entry - Free memory for execve().
- *
- * @ee: Pointer to "struct ccs_execve_entry".
- */
-static inline void ccs_free_execve_entry(struct ccs_execve_entry *ee)
-{
-	if (!ee)
-		return;
-	spin_lock(&ccs_execve_list_lock);
-	list_del(&ee->list);
-	ccs_in_execve_counter--;
-	spin_unlock(&ccs_execve_list_lock);
-	kfree(ee->handler_path);
-	kfree(ee->tmp);
-	kfree(ee->dump.data);
-	ccs_read_unlock(ee->reader_idx);
-	kfree(ee);
-}
-
 /**
  * ccs_try_alt_exec - Try to start execute handler.
  *
@@ -965,6 +914,9 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 	struct task_struct *task = current;
 
 	/* Close the requested program's dentry. */
+	ee->obj.path1.dentry = NULL;
+	ee->obj.path1.mnt = NULL;
+	ee->obj.validate_done = false;
 	allow_write_access(bprm->file);
 	fput(bprm->file);
 	bprm->file = NULL;
@@ -1091,6 +1043,8 @@ static int ccs_try_alt_exec(struct ccs_execve_entry *ee)
 		retval = PTR_ERR(filp);
 		goto out;
 	}
+	ee->obj.path1.dentry = filp->f_dentry;
+	ee->obj.path1.mnt = filp->f_vfsmnt;
 	bprm->file = filp;
 	bprm->filename = ee->handler_path;
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
@@ -1155,7 +1109,7 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 		   struct ccs_page_dump *dump)
 {
 	struct page *page;
-	/* dump->data is released by ccs_free_execve_entry(). */
+	/* dump->data is released by ccs_finish_execve(). */
 	if (!dump->data) {
 		dump->data = kzalloc(PAGE_SIZE, GFP_KERNEL);
 		if (!dump->data)
@@ -1168,6 +1122,9 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 #elif defined(RHEL_MAJOR) && RHEL_MAJOR == 5 && defined(RHEL_MINOR) && RHEL_MINOR >= 3 && defined(CONFIG_MMU)
 	if (get_user_pages(current, bprm->mm, pos, 1, 0, 1, &page, NULL) <= 0)
 		return false;
+#elif defined(AX_MAJOR) && AX_MAJOR == 3 && defined(AX_MINOR) && AX_MINOR >= 2 && defined(CONFIG_MMU)
+	if (get_user_pages(current, bprm->mm, pos, 1, 0, 1, &page, NULL) <= 0)
+		return false;
 #else
 	page = bprm->page[pos / PAGE_SIZE];
 #endif
@@ -1188,6 +1145,8 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 	put_page(page);
 #elif defined(RHEL_MAJOR) && RHEL_MAJOR == 5 && defined(RHEL_MINOR) && RHEL_MINOR >= 3 && defined(CONFIG_MMU)
 	put_page(page);
+#elif defined(AX_MAJOR) && AX_MAJOR == 3 && defined(AX_MINOR) && AX_MINOR >= 2 && defined(CONFIG_MMU)
+	put_page(page);
 #endif
 	return true;
 }
@@ -1196,26 +1155,45 @@ bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
  * ccs_start_execve - Prepare for execve() operation.
  *
  * @bprm: Pointer to "struct linux_binprm".
+ * @eep:  Pointer to "struct ccs_execve_entry *".
  *
  * Returns 0 on success, negative value otherwise.
  */
-int ccs_start_execve(struct linux_binprm *bprm)
+int ccs_start_execve(struct linux_binprm *bprm, struct ccs_execve_entry **eep)
 {
 	int retval;
 	struct task_struct *task = current;
-	struct ccs_execve_entry *ee = ccs_allocate_execve_entry();
+	struct ccs_execve_entry *ee;
+	*eep = NULL;
 	if (!ccs_policy_loaded)
 		ccs_load_policy(bprm->filename);
+	ee = kzalloc(sizeof(*ee), GFP_KERNEL);
 	if (!ee)
 		return -ENOMEM;
+	ee->tmp = kzalloc(CCS_EXEC_TMPSIZE, GFP_KERNEL);
+	if (!ee->tmp) {
+		kfree(ee);
+		return -ENOMEM;
+	}
+	ee->reader_idx = ccs_read_lock();
+	/* ee->dump->data is allocated by ccs_dump_page(). */
+	ee->previous_domain = task->ccs_domain_info;
+	/* Clear manager flag. */
+	task->ccs_flags &= ~CCS_TASK_IS_POLICY_MANAGER;
+	/* Tell GC that I started execve(). */
+	task->ccs_flags |= CCS_TASK_IS_IN_EXECVE;
+	/*
+	 * Make task->ccs_flags visible to GC before changing
+	 * task->ccs_domain_info .
+	 */
+	smp_mb();
+	*eep = ee;
 	ccs_init_request_info(&ee->r, NULL, CCS_MAC_FILE_EXECUTE);
 	ee->r.ee = ee;
 	ee->bprm = bprm;
 	ee->r.obj = &ee->obj;
 	ee->obj.path1.dentry = bprm->file->f_dentry;
 	ee->obj.path1.mnt = bprm->file->f_vfsmnt;
-	/* Clear manager flag. */
-	task->ccs_flags &= ~CCS_TASK_IS_POLICY_MANAGER;
 	if (ccs_find_execute_handler(ee, CCS_TYPE_EXECUTE_HANDLER)) {
 		retval = ccs_try_alt_exec(ee);
 		if (!retval)
@@ -1244,11 +1222,8 @@ int ccs_start_execve(struct linux_binprm *bprm)
 	retval = ccs_environ(ee);
 	if (retval < 0)
 		goto out;
-	task->ccs_flags |= CCS_CHECK_READ_FOR_OPEN_EXEC;
 	retval = 0;
  out:
-	if (retval)
-		ccs_finish_execve(retval);
 	return retval;
 }
 
@@ -1256,35 +1231,35 @@ int ccs_start_execve(struct linux_binprm *bprm)
  * ccs_finish_execve - Clean up execve() operation.
  *
  * @retval: Return code of an execve() operation.
+ * @ee:     Pointer to "struct ccs_execve_entry".
  *
  * Caller holds ccs_read_lock().
  */
-void ccs_finish_execve(int retval)
+void ccs_finish_execve(int retval, struct ccs_execve_entry *ee)
 {
 	struct task_struct *task = current;
-	struct ccs_execve_entry *ee = NULL;
-	struct ccs_execve_entry *p;
-	task->ccs_flags &= ~CCS_CHECK_READ_FOR_OPEN_EXEC;
-	spin_lock(&ccs_execve_list_lock);
-	list_for_each_entry(p, &ccs_execve_list, list) {
-		if (p->task != task)
-			continue;
-		ee = p;
-		break;
-	}
-	spin_unlock(&ccs_execve_list_lock);
 	if (!ee)
 		return;
 	if (retval < 0) {
 		task->ccs_domain_info = ee->previous_domain;
-		goto out;
+		/*
+		 * Make task->ccs_domain_info visible to GC before changing
+		 * task->ccs_flags .
+		 */
+		smp_mb();
+	} else {
+		/* Mark the current process as execute handler. */
+		if (ee->handler)
+			task->ccs_flags |= CCS_TASK_IS_EXECUTE_HANDLER;
+		/* Mark the current process as normal process. */
+		else
+			task->ccs_flags &= ~CCS_TASK_IS_EXECUTE_HANDLER;
 	}
-	/* Mark the current process as execute handler. */
-	if (ee->handler)
-		task->ccs_flags |= CCS_TASK_IS_EXECUTE_HANDLER;
-	/* Mark the current process as normal process. */
-	else
-		task->ccs_flags &= ~CCS_TASK_IS_EXECUTE_HANDLER;
- out:
-	ccs_free_execve_entry(ee);
+	/* Tell GC that I finished execve(). */
+	task->ccs_flags &= ~CCS_TASK_IS_IN_EXECVE;
+	ccs_read_unlock(ee->reader_idx);
+	kfree(ee->handler_path);
+	kfree(ee->tmp);
+	kfree(ee->dump.data);
+	kfree(ee);
 }
