@@ -3,28 +3,19 @@
  *
  * Copyright (C) 2005-2009  NTT DATA CORPORATION
  *
- * Version: 1.7.0   2009/09/06
+ * Version: 1.7.1   2009/11/11
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
  *
  */
 
+#include "internal.h"
 #include <linux/version.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 #define __KERNEL_SYSCALLS__
-#endif
-#include "internal.h"
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 #include <linux/unistd.h>
 #endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
-#include <linux/namei.h>
-#include <linux/mount.h>
-static const int ccs_lookup_flags = LOOKUP_FOLLOW;
-#else
-static const int ccs_lookup_flags = LOOKUP_FOLLOW | LOOKUP_POSITIVE;
-#endif
 
 /* Path to the policy loader. The default is /sbin/ccs-init. */
 static const char *ccs_loader;
@@ -58,19 +49,15 @@ static bool ccs_policy_loader_exists(void)
 	 * policies are not loaded yet.
 	 * Thus, let do_execve() call this function everytime.
 	 */
-	struct nameidata nd;
+	struct path path;
 	if (!ccs_loader)
-		ccs_loader = "/sbin/ccs-init";
-	if (path_lookup(ccs_loader, ccs_lookup_flags, &nd)) {
+		ccs_loader = CONFIG_CCSECURITY_DEFAULT_LOADER;
+	if (ccs_get_path(ccs_loader, &path)) {
 		printk(KERN_INFO "Not activating Mandatory Access Control now "
 		       "since %s doesn't exist.\n", ccs_loader);
 		return false;
 	}
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
-	path_put(&nd.path);
-#else
-	path_release(&nd);
-#endif
+	path_put(&path);
 	return true;
 }
 
@@ -114,14 +101,8 @@ void ccs_load_policy(const char *filename)
 {
 	if (ccs_policy_loaded)
 		return;
-	/*
-	 * Check filename is /sbin/init or /sbin/ccs-start.
-	 * /sbin/ccs-start is a dummy filename in case where /sbin/init can't
-	 * be passed.
-	 * You can create /sbin/ccs-start by "ln -s /bin/true /sbin/ccs-start".
-	 */
 	if (strcmp(filename, "/sbin/init") &&
-	    strcmp(filename, "/sbin/ccs-start"))
+	    strcmp(filename, CONFIG_CCSECURITY_ALTERNATIVE_TRIGGER))
 		return;
 	if (!ccs_policy_loader_exists())
 		return;
@@ -177,7 +158,7 @@ void ccs_load_policy(const char *filename)
 		spin_unlock_irq(&task->sigmask_lock);
 	}
 #endif
-	printk(KERN_INFO "CCSecurity: 1.7.0+   2009/09/06\n");
-	printk(KERN_INFO "Mandatory Access Control activated.\n");
+	printk(KERN_INFO "CCSecurity: 1.7.1   2009/11/11\n");
 	ccs_check_profile();
+	printk(KERN_INFO "Mandatory Access Control activated.\n");
 }
